package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.core.api.iservice.busi.*;
import cn.estsh.i3plus.platform.common.tool.ExcelTool;
import cn.estsh.i3plus.platform.common.tool.FileTool;
import cn.estsh.i3plus.platform.common.tool.ZipTool;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.platform.platbean.SysLogOperate;
import cn.estsh.impp.framework.base.controller.CoreBaseController;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.persistence.EntityManager;
import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @Description : 操作日志服务
 * @Reference :
 * @Author : yunhao
 * @Date : 2018-10-25 14:08
 * @Modify :
 **/
@RestController
@Api(description = "操作日志")
@RequestMapping(PlatformConstWords.BASE_URL + "/sys-log-operate")
public class SysLogOperateController extends CoreBaseController {
    public static final Logger LOGGER = LoggerFactory.getLogger(SysLogOperateController.class);

    @Autowired
    private ISysLogOperateService logOperateService;

    @Autowired
    private ISysDictionaryService sysDictionaryService;

    @Autowired
    private ISysConfigService sysConfigService;

    @Autowired
    private ISysFileService sysFileService;

    @Autowired
    private ISysMessageService sysMessageService;

    @Autowired
    private EntityManager entityManager;

    @Resource(name="redisRes")
    private ImppRedis redisRes;

    /**
     * 添加操作日志
     * @param logOperate 操作日志
     * @return 处理结果
     */
    @PostMapping(value = "/insert")
    @ApiOperation(value = "添加操作日志",notes = "添加操作日志")
    public ResultBean insertSysLogOperateController(SysLogOperate logOperate){
        try {
            logOperateService.insertSysLogOperate(logOperate);
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据id删除操作日志
     * @param id 操作日志id
     * @return 处理结果
     */
    @DeleteMapping(value = "/delete")
    @ApiOperation(value = "根据id删除操作日志",notes = "根据id删除操作日志")
    public ResultBean deleteSysLogOperateById(String id){
        try {
            // 条件验证
            ValidatorBean.checkNotNull(id,"id不能为空！");

            logOperateService.deleteSysLogOperateById(Long.parseLong(id));
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询全部操作日志
     * @return 处理结果
     */
    @GetMapping(value = "/list")
    @ApiOperation(value = "查询全部操作日志",notes = "查询全部操作日志")
    public ResultBean findSysLogOperateAll(){
        try {
            List logOperateList = logOperateService.listSysLogOperate();
            return ResultBean.success("操作成功").setResultList(logOperateList).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据id查询操作日志
     * @param id 操作日志
     * @return 处理结果
     */
    @GetMapping(value = "/get/{id}")
    @ApiOperation(value = "根据id查询操作日志",notes = "根据id查询操作日志")
    public ResultBean getSysLogOperateById(@PathVariable("id") String id){
        try{
            SysLogOperate logOperate = logOperateService.getSysLogOperateById(Long.parseLong(id));
            if (logOperate != null) {
                return ResultBean.success("操作成功")
                        .setResultObject(logOperate)
                        .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
            }else {
                return ResultBean.fail("操作日志不存在")
                        .setCode(ResourceEnumUtil.MESSAGE.EMPTY.getCode());
            }
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 操作日志复杂查询，分页，排序
     * @param logOperate 操作日志
     * @param pager 分页数据
     * @return 处理结果
     */
    @GetMapping(value = "/query")
    @ApiOperation(value = "操作日志复杂查询，分页，排序",notes = "操作日志复杂查询，分页，排序")
    public ResultBean querySysLogOperateByPager(SysLogOperate logOperate, Pager pager){
        try {
            ListPager logSystemList = logOperateService.querySysLogOperateByPager(logOperate, pager);
            return ResultBean.success("操作成功").setListPager(logSystemList).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 获取系统操作日志字段
     * @return 处理结果
     */
    @GetMapping("/get-col")
    @ApiOperation(value = "获取系统操作日志字段")
    public ResultBean getSysLogOperateCol(){
        try {
            return  ResultBean.success("导出成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultMap(ExcelTool.getColName(SysLogOperate.class));
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 导出日志
     * @param sysLogOperate 查询条件
     * @param colName 导出列名
     * @param pageSize 页大小
     * @param isZip 是否发送邮件
     * @return 处理结果
     */
    @GetMapping(value = "/export")
    @ApiOperation(value = "导出操作日志")
    public ResultBean exportSysLogOperate(SysLogOperate sysLogOperate, String[] colName, int pageSize, boolean isZip) {
        ListPager listPager;
        List result = new ArrayList();
        File zipFile = null;
        File excelDir = null;
        File excelFile;
        try {
            startMultiService();

            Pager pager = new Pager();
            pager.setCurrentPage(1);
            // 判断是否超过excel最大导出数量设定值
            int maxPageSize = Integer.parseInt(sysConfigService.getSysConfigByCode(PlatformConstWords.EXCEL_EXPORT_MAX_ROW).getConfigValue());
            if (pageSize == 0 || pageSize >= maxPageSize) {
                pager.setPageSize(maxPageSize);
            } else {
                pager.setPageSize(pageSize);
            }

            // 数据校验
            if(colName==null || colName.length == 0){
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.IMPP.getCode())
                        .setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode())
                        .setErrorDetail("导出列不能为空")
                        .build();
            }

            excelDir = new File(System.getProperty("java.io.tmpdir") + File.separator + new Date().getTime());
            excelDir.mkdir();

            ExcelTool excelTool = new ExcelTool(entityManager, redisCore);
            do {
                listPager = logOperateService.querySysLogOperateByPager(sysLogOperate, pager);
                pager = listPager.getObjectPager();

                // 将excel导出至临时文件夹并上传
                excelFile = new File(excelDir + File.separator + SysLogOperate.class.getSimpleName() + pager.getCurrentPage() + ".xls");
                excelFile.createNewFile();

                excelTool.exportData(excelFile, listPager.getObjectList(), SysLogOperate.class, colName);
                sysFileService.uploadFile(excelFile);

                pager.setCurrentPage(pager.getCurrentPage() + 1);
            } while (pager.getCurrentPage() <= pager.getTotalPages());

            // 将所有excel文件打包
            if(isZip){
                zipFile = ZipTool.zipFile(null, excelDir);
                result.add(sysFileService.uploadFile(zipFile));
                sysMessageService.doSendSysMessage(ExcelTool.sendStationLetter(result,getSessionUser().getUser().getId()));
                return ResultBean.success("导出成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(result);
            } else {
                return ResultBean.success("导出成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(result);
            }
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        } finally {
            // 清理临时文件文件
            if(excelDir == null) {
                FileTool.deleteFile(excelDir.getPath());
            }
        }
    }
}
