package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysMessageService;
import cn.estsh.i3plus.core.apiservice.controller.base.BaseMessageController;
import cn.estsh.i3plus.platform.common.tool.StringTool;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ImppEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.model.platform.MessagePagerModel;
import cn.estsh.i3plus.pojo.model.platform.RefUserMessagePagerModel;
import cn.estsh.i3plus.pojo.platform.bean.SysMessage;
import cn.estsh.i3plus.pojo.platform.bean.SysRefUserMessage;
import cn.estsh.impp.framework.base.controller.CoreBaseController;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 * @Description : 消息管理服务
 * @Reference :
 * @Author : yunhao
 * @Date : 2018-10-25 19:03
 * @Modify :
 **/
@RestController
@RequestMapping("/impp")
@Api(description = "消息管理服务")
public class SysMessageController extends CoreBaseController {
    public static final Logger LOGGER = LoggerFactory.getLogger(SysMessageController.class);

    @Autowired
    private ISysMessageService sysMessageService;

    @Autowired
    private BaseMessageController baseSysMessage;

    /*************************************************** 平台系统消息 **************************************************/

    /**
     * CORE - 新增消息
     * @param sysMessage 系统消息
     * @return 处理结果
     */
    @PostMapping(value = "/operate/sys-message/insert")
    @ApiOperation(value = "新增消息",notes = "新增消息")
    public ResultBean<SysMessage> insertSysMessage(@RequestBody SysMessage sysMessage){
        sysMessage = sysMessage == null ? new SysMessage() : sysMessage;
        sysMessage.setMessageSoftType(CommonEnumUtil.SOFT_TYPE.CORE.getValue());

        return baseSysMessage.basaeInsertSysMessage(sysMessage);
    }

    /**
     * CORE - 删除消息
     * @param id 消息id
     * @return 处理结果
     */
    @DeleteMapping("/operate/sys-message/delete/{id}")
    @ApiOperation(value = "根据id删除消息",notes = "根据id删除消息")
    public ResultBean deleteSysMessageById(@PathVariable("id") String id){
        return baseSysMessage.baseDeleteSysMessageById(id);
    }

    /**
     * CORE - 修改消息
     * @param sysMessage 系统消息
     * @return 处理结果
     */
    @PutMapping("/operate/sys-message/update")
    @ApiOperation(value = "修改信息",notes = "修改信息")
    public ResultBean updateSysMessage(SysMessage sysMessage){
        sysMessage = sysMessage == null ? new SysMessage() : sysMessage;
        sysMessage.setMessageSoftType(CommonEnumUtil.SOFT_TYPE.CORE.getValue());
        return baseSysMessage.baseUpdateSysMessage(sysMessage);
    }

    /**
     * 查询所有消息
     * @return 处理结果
     */
    @GetMapping(value = "/operate/sys-message/list")
    @ApiOperation(value = "查询全部消息",notes = "查询全部消息")
    public ResultBean findSysMessageAll(){
        try {
            List<SysMessage> sysMessageList = sysMessageService.listSysMessage();
            return  ResultBean.success("操作成功").setResultList(sysMessageList).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * CORE - 根据id查询消息
     * @param idStr 消息id
     * @return 处理结果
     */
    @GetMapping(value = "/operate/sys-message/get/{id}")
    @ApiOperation(value = "根据id查询消息",notes = "根据id查询消息")
    public ResultBean<SysMessage> getSysMessageById(@PathVariable("id") String idStr){
        return baseSysMessage.baseGetSysMessageById(idStr);
    }

    /**
     * CORE - 分页带条件查询
     * @param messagePagerModel 分页以及条件信息
     * @return 处理结果
     */
    @PostMapping(value = "/operate/sys-message/query")
    @ApiOperation(value = "消息复杂查询，分页，排序",notes = "消息复杂查询，分页，排序")
    public ResultBean<SysMessage> querySysMessageByPager(@RequestBody MessagePagerModel messagePagerModel){
        return baseSysMessage.baseQuerySysMessageByPager(messagePagerModel);
    }

    /**
     * CORE - 批量删除
     * @param ids ids数组
     * @return 处理结果
     */
    @DeleteMapping("/operate/sys-message/batch-delete")
    @ApiOperation(value = "批量删除消息",notes = "批量删除消息")
    public ResultBean deleteSysMessageByIds(String[] ids){
        return baseSysMessage.baseDeleteSysMessageByIds(ids);
    }

    /*********************************************** 平台用户系统消息 *********************************************/

    /**
     * CORE - 收件箱查询
     * @param sysRefUserMessage 查询条件
     * @param pager 分页信息
     * @return 处理结果
     */
    @GetMapping(value = "/operate/user-message/query-inbox")
    @ApiOperation(value = "查询用户收件箱，分页，排序")
    public ResultBean<SysMessage> queryInboxByPager(SysRefUserMessage sysRefUserMessage, Pager pager){
        try {
            // 设置消息类型为站内信
            sysRefUserMessage.setMessageTypeRdd(ImppEnumUtil.MESSAGE_TYPE.LETTER.getValue());
            sysRefUserMessage.setReceiverId(getSessionUser().getUser().getId());

            ListPager userMessageList = sysMessageService.querySysRefUserMessageByPager(sysRefUserMessage,pager);
            return  ResultBean.success("操作成功").setListPager(userMessageList).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * CORE - 发件箱查询
     * @param sysMessage 查询条件
     * @param pager 分页信息
     * @return 处理结果
     */
    @GetMapping(value = "/operate/user-message/query-outbox")
    @ApiOperation(value = "查询用户发件箱，分页，排序")
    public ResultBean<SysMessage> queryOutboxByPager(SysMessage sysMessage,Pager pager){
        try {
            // 发件人为当前用户
            sysMessage.setMessageType(ImppEnumUtil.MESSAGE_TYPE.LETTER.getValue());
            sysMessage.setMessageSenderId(getSessionUser().getUser().getId());

            ListPager sysMessageListPager = sysMessageService.querySysMessageByPager(sysMessage,pager);
            return  ResultBean.success("操作成功").setListPager(sysMessageListPager).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * CORE - 查询未读消息
     * @return 处理结果
     */
    @GetMapping(value = "/operate/user-message/find-unread")
    @ApiOperation(value = "查看用户未读站内信")
    public ResultBean<SysRefUserMessage> findUnreadUserMessage(){
        try {
            List<SysRefUserMessage> userMessageList = sysMessageService.findSysRefUserMessageByUserIdAndStatus(
                    getSessionUser().getUser().getId(), ImppEnumUtil.MESSAGE_STATUS.UNREAD.getValue());
            return  ResultBean.success("操作成功").setResultList(userMessageList).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * CORE - 根据用户消息id查询消息
     * @param idStr 消息id
     * @return 处理结果
     */
    @GetMapping(value = "/operate/user-message/get/{id}")
    @ApiOperation(value = "根据用户消息id查询消息",notes = "根据用户消息id查询消息")
    public ResultBean<SysMessage> getUserMessageById(@PathVariable("id") String idStr){
        return baseSysMessage.baseGetUserMessageById(idStr);
    }

    /**
     * CORE - 批量切换消息状态
     * @param ids ids数组
     * @param status 状态
     * @return 处理结果
     */
    @PutMapping("/operate/user-message/batch-status")
    @ApiOperation(value = "批量切换消息状态",notes = "批量切换消息状态")
    public ResultBean updateSysMessageStatusByIds(String[] ids,Integer status){
        return baseSysMessage.baseUpdateSysMessageStatusByIds(ids, status);
    }

    /**
     * 根据id删除用户消息
     * @param idStr 用户表消息id
     * @return 处理结果
     */
    @DeleteMapping(value = "/operate/user-message/delete-inbox/{id}")
    @ApiOperation(value = "根据id删除用户消息")
    public ResultBean deleteUserMessageById(@PathVariable("id") String idStr){
        try {
            sysMessageService.deleteSysRefUserMessageById(Long.parseLong(idStr));
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据id批量删除用户消息
     * @param idsStr 用户消息id数组
     * @return 处理结果
     */
    @DeleteMapping(value = "/operate/user-message/batch-delete-inbox")
    @ApiOperation(value = "根据id批量删除用户消息")
    public ResultBean deleteUserMessageByIds(String[] idsStr){
        try {
            sysMessageService.deleteSysRefUserMessageByIds(StringTool.getArrayLong(idsStr));
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 用户新增消息
     * @param sysMessage 系统消息
     * @return 处理结果
     */
    @PostMapping(value = "/operate/user-message/insert")
    @ApiOperation(value = "用户新增消息",notes = "用户新增消息")
    public ResultBean insertUserMessage(SysMessage sysMessage) {
        try {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            // 条件校验
            ValidatorBean.beginValid(sysMessage)
                    .notNull("messageTitle",sysMessage.getMessageTitle())
                    .notNull("messageContent",sysMessage.getMessageContent())
                    .notNull("messageReceiversId",sysMessage.getMessageReceiversId());

            // 设置发件人名称
            sysMessage.setMessageType(ImppEnumUtil.MESSAGE_TYPE.LETTER.getValue());
            sysMessage.setMessageSenderId(getSessionUser().getUser().getId());
            sysMessage.setMessageSenderNameRdd(getSessionUser().getUserName());
            sysMessage.setMessageSendTime(sdf.format(new Date()));
            sysMessage.setMessageContentType(ImppEnumUtil.MESSAGE_TYPE_CONTENT.HTML.getValue());
            sysMessage.setMessageReceiverType(ImppEnumUtil.MESSAGE_RECEIVER_TYPE.INTERNAL.getValue());

            sysMessage.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());

            sysMessageService.doSendSysMessage(sysMessage);
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 用户消息复杂查询，分页，排序
     * @param refUserMessagePagerModel 查询条件及分页信息
     * @return 处理结果
     */
    @PostMapping(value = "/operate/user-message/query")
    @ApiOperation(value = "消息复杂查询，分页，排序",notes = "消息复杂查询，分页，排序")
    public ResultBean<SysRefUserMessage> querySysRefUserMessageByPager(@RequestBody RefUserMessagePagerModel refUserMessagePagerModel){
        return baseSysMessage.baseQuerySysRefUserMessageByPager(refUserMessagePagerModel);
    }

    /**
     * 用户消息删除
     * @param idStr 用户消息id
     * @return 处理结果
     */
    @DeleteMapping(value = "/operate/user-message/delete/{id}")
    @ApiOperation(value = "消息复杂查询，分页，排序",notes = "消息复杂查询，分页，排序")
    public ResultBean<SysRefUserMessage> deleteSysRefUserMessageById(@PathVariable("id") String idStr){
        return baseSysMessage.baseDeleteSysRefUserMessageById(idStr);
    }
}
